clc
clear; close all
addpath common

compile

dataSize=[100,250,500];
th=2;

for i=1:length(dataSize)
    fprintf('Running BnB for %d points.\n',dataSize(i));
    
    % Generate random data with noise and outliers
    Y = randpts(dataSize(i),100);
    X = Y + 1*randn(dataSize(i),3);
    outIdx = randsample(dataSize(i), floor(.25*dataSize(i)));
    for k=outIdx'
        X(k,:) = X(k,:)*randrot();
    end
    
    % Initialise
    R_init = vrrotvec2mat([0 0 1 rand()*pi]);
    X = X*R_init;
    
    % Plot input points
    X = pointCloud(X);
    Y = pointCloud(Y);
    
    figure
    subplot(1,2,1);
    pcsetcolor(X,'r');
    pcsetcolor(Y,'b');
    pcshow(X, 'markerSize', 100)
    hold on
    pcshow(Y, 'markerSize', 100)
    hold off
    title('Random point clouds')
    drawnow
    
    % Execute BnB
    % warm up
    if i==1
        [~,~] = rot1(X.Location',Y.Location',th,1,10,true);
    end

    bnb = struct();
    tic
    [bnb.q, bnb.angle] = rot1(X.Location',Y.Location',th,0,0,true);
    bnb.time = toc;
    bnb.R = vrrotvec2mat([0 0 1 bnb.angle])';
    
    fprintf('Number of points........... %d\n', dataSize(i));
    fprintf('Time (s)................... %f\n', bnb.time);
    fprintf('Output obj value........... %d\n', bnb.q);
    fprintf('Output rotation matrix:');
    display(bnb.R);
    
    % Plot found rotation over all points.
    subplot(1,2,2);
    pcshow(pctransform(X,affine3d([bnb.R [0 0 0]'; 0 0 0 1])), 'markerSize', 100)
    hold on
    pcshow(Y, 'markerSize', 100)
    hold off
    title('Opt rotation')
    drawnow
end



