/////////////////////////////////////////////////////////////////////////////
//
//              R O T A T I O N   S E A R C H
//
// This package contains the source code which implements the
// BnB rotation search algorithm proposed in
//
// T.-J. Chin, A. Parra Bustos, M. S. Brown and D. Suter
// Fast rotation search for real-time interactive point cloud registration
// In ACM SIGGRAPH Symposium on Interactive 3D Graphics and Games (I3D) 2014.
//
// Copyright (c) 2016 Alvaro PARRA BUSTOS (aparra@cs.adelaide.edu.au.)
// School of Computer Science, The University of Adelaide, Australia
// The Australian Center for Visual Technologies
// http://cs.adelaide.edu.au/~aparra
// Please acknowledge the authors by citing the above paper in any academic
// publications that have made use of this package or part of it.
//
/////////////////////////////////////////////////////////////////////////////


#ifndef REG_ARC_EVALUATOR_
#define REG_ARC_EVALUATOR_

#include "reg_common.h"
#include "data_indexation.h"
#include "state.h"
#include "interval_tree.h"

namespace reg {
    namespace search {
        
        typedef RotationSearchSpaceRegion1DOF SSR;
        
        class ArcEvaluator : public DataIndexation<SSR>
        {
        public:
            
            ArcEvaluator( const Matrix3X &X, const Matrix3X &Y, const Vector &TH );
            ~ArcEvaluator();
            
            int sweep();
            int size() const;
            
            int evalUpperBound(SSR ssr, int lwbnd) const;
            int evalUpperBound(SSR ssr, int lwbnd,
                               int *matchList, int matchListSize,
                               std::vector<bool> &matches) const;
            
            int evalLowerBound(SSR ssr) const;
            int evalLowerBound(SSR ssr, int *matchList, int matchListSize) const;
            int evalLowerBound(SSR ssr, int *matchList, int matchListSize,
                               std::vector<bool> &matches) const;
            
        private:
            
            const Matrix3X &M_in;
            const Matrix3X &B_in;
            const Vector &TH_in;
            
            IntervalTree **tree_array;
            
            int _size;
        };
        
        
    } // End namespace sarch
} // End namespace reg

#endif