#include "reg_common.h"
#include "search.h"
#include "rot1_evaluator.h"
#include "registration1.h"

using namespace reg::search;


int reg::search::rot1(const Matrix3X &X, const Matrix3X &Y,
                      double th, int lwbnd, int gap, double &outAngle)
{
    mxAssert(th>0 && gap>=0 && lwbnd>=0, "");
    typedef RotationSearchSpaceRegion1DOF SSR;

    SSR guessAndResult(0, TWOPI);
    const size_t numOfPts = X.cols();
    mxAssert(numOfPts>0,"");

    Vector TH(numOfPts);
    for (int i=0; i<numOfPts; i++)
    {
        TH.x[i]=th;
    }
    
    DataIndexation<SSR> *dsi = new ArcEvaluator(X,Y,TH);
    dsi->sweep();

    const int buckets = MAX(numOfPts/10, 10);
    
    int q = bnb_search_table<SSR,2>(*dsi, lwbnd, gap, buckets, guessAndResult);
    delete dsi;

    outAngle = .5*(guessAndResult.lw + guessAndResult.up);

    return q;
}


int reg::search::rot1_ml(const Matrix3X &X, const Matrix3X &Y,
                      double th, int lwbnd, int gap, double &outAngle)
{
    mxAssert(th>0 && gap>=0 && lwbnd>=0, "");
    typedef RotationSearchSpaceRegion1DOF SSR;
    
    SSR guessAndResult(0, TWOPI);
    const size_t numOfPts = X.cols();
    mxAssert(numOfPts>0,"");
    
    Vector TH(numOfPts);
    for (int i=0;i<numOfPts;i++)
    {
        TH.x[i]=th;
    }
    
    DataIndexation<SSR> *dsi = new ArcEvaluator(X,Y,TH);
    dsi->sweep();
    
    const int buckets = MAX(numOfPts/10, 10);
    
    int q = bnb_search_ml_table<SSR,2>(*dsi, lwbnd, gap, buckets, guessAndResult);
    delete dsi;
    
    outAngle = .5*(guessAndResult.lw + guessAndResult.up);
    
    return q;
}
